"""
Graphviz extension for Markdown (e.g. for mkdocs) :
Renders the output inline, eliminating the need to configure an output
directory.

Supports outputs types of SVG and PNG. The output will be taken from the
filename specified in the tag, if given, or. Example:

in SVG:

```dot
digraph G {
    rankdir=LR
    Earth [peripheries=2]
    Mars
    Earth -> Mars
}
```

```graphviz dot attack_plan.svg
digraph G {
    rankdir=LR
    Earth [peripheries=2]
    Mars
    Earth -> Mars
}
```

in PNG:

```graphviz dot attack_plan.png
digraph G {
    rankdir=LR
    Earth [peripheries=2]
    Mars
    Earth -> Mars
}
```

Requires the graphviz library (http://www.graphviz.org/) and python 3

Inspired by jawher/markdown-dot (https://github.com/jawher/markdown-dot)
Forked from  cesaremorel/markdown-inline-graphviz (https://github.com/cesaremorel/markdown-inline-graphviz)
"""

import os
import re
import markdown
import subprocess
import base64
import shlex
from random import randint
from .htmlColors import HTML_COLORS

# Global vars
BLOCK_RE_TEX2SVG = re.compile(
    r'^[ 	]*```tex2svg[ ]* (?P<command>\w+)\s+(?P<filename>[^\s]+)\s*\n(?P<content>.*?)```\s*$',
    re.MULTILINE | re.DOTALL
    )

BLOCK_RE_VARTABLE = re.compile(
        r'^[ 	]*```vartable\n(?P<content>.*?)```\s*$',
    re.MULTILINE | re.DOTALL)

TEX2SVG_COMMAND = 0
tex2svgVersion = "0.0.4"

# Command whitelist
SUPPORTED_COMMANDS = ['tex2svg', 'vartable']
AUTHORIZED_CONFIGS = ["vartable", "priority"]

# DEFAULT COLOR OF TEXTS, BACKGROUNDS for LABELS and for FV - Forbidden Values (Valeurs Interdites - VI)
DEFAULT_COLOR = '789ABC'
DEFAULT_LIGHT_COLOR = '000000'
DEFAULT_LIGHT_BGLABEL = 'FFFFFF'
DEFAULT_LIGHT_BGVI = 'FFFFFF'

DEFAULT_DARK_COLOR = 'FFFFFF'
DEFAULT_DARK_BGLABEL = '2E303E'
DEFAULT_DARK_BGVI = '2E303E'

DEFAULT_VARTABLE_DICT = {
    'light': {
        'color': f'{DEFAULT_LIGHT_COLOR}',
        'bglabel': f'{DEFAULT_LIGHT_BGLABEL}', 
        'bgvi': f'{DEFAULT_LIGHT_BGVI}'}, 
    'dark': {
        'color': f'{DEFAULT_DARK_COLOR}', 
        'bglabel': f'{DEFAULT_DARK_BGLABEL}', 
        'bgvi': f'{DEFAULT_DARK_BGVI}'
        }
    }

DEFAULT_COLOR = DEFAULT_COLOR.lower()
DEFAULT_LIGHT_COLOR = DEFAULT_LIGHT_COLOR.lower()
# DEFAULT_DARKTHEME_COLOR = DEFAULT_DARKTHEME_COLOR.lower()
DEFAULT_PRIORITY = '75'

inlineMode = True
DEFAULT_PACKAGES = ['amsmath', 'tkz-tab', 'amssymb']

DEBUG = True

ESC_CHAR = {
    '$': r"\$",
    '*': r"\*",
    '^': r"\^",
}

class MkdocsTex2SvgExtension(markdown.Extension):

    def __init__(self, **kwargs):
        self.config = {
            'vartable' :        [DEFAULT_VARTABLE_DICT, 'Default Vartable Dict'],
            'priority' :        [DEFAULT_PRIORITY, 'Default Priority for this Extension']
        }
        super(MkdocsTex2SvgExtension, self).__init__(**kwargs)

    def extendMarkdown(self, md):
        """ Add MkdocsTex2SvgPreprocessor to the Markdown instance. """
        md.registerExtension(self)
        md.preprocessors.register(MkdocsTex2SvgPreprocessor(md, self.config), 'tex2svg_block', int(self.config['priority'][0]))

class MkdocsTex2SvgPreprocessor(markdown.preprocessors.Preprocessor):
    def __init__(self, md, config):
        super(MkdocsTex2SvgPreprocessor, self).__init__(md)
        self.config = config
        self.formatConfigOptions()

    def formatConfigOptions(self):
        print("self.config=", self.config)
        for entry in self.config.keys():
            if entry == "vartable": # format vartable options
                colorLight = self.config[entry][0]["light"]["color"] = self.config[entry][0]["light"]["color"].lower()
                bgLabelLight = self.config[entry][0]["light"]["bglabel"] = self.config[entry][0]["light"]["bglabel"].lower()
                bgVILight = self.config[entry][0]["light"]["bgvi"] = self.config[entry][0]["light"]["bgvi"].lower()

                colorDark = self.config[entry][0]["dark"]["color"] = self.config[entry][0]["dark"]["color"].lower()
                bgLabelDark = self.config[entry][0]["dark"]["bglabel"] = self.config[entry][0]["dark"]["bglabel"].lower()
                bgVIDark = self.config[entry][0]["dark"]["bgvi"] = self.config[entry][0]["dark"]["bgvi"].lower()

                # Set Colors of Texts & Borders
                if colorLight in HTML_COLORS.keys(): # HTML HEXA COLOR, just for information
                    colorLight = self.config[entry][0]["light"]["color"]
                elif len(self.config[entry][0]["light"]["color"]) in [3, 6, 8]:
                    colorLight = self.config[entry][0]["light"]["color"] = "#"+str(self.config[entry][0]["light"]["color"])

                if colorDark in HTML_COLORS.keys(): # HTML HEXA COLOR, just for information
                    colorDark = self.config[entry][0]["dark"]["color"]
                elif len(self.config[entry][0]["dark"]["color"]) in [3, 6, 8]:
                    colorDark = self.config[entry][0]["dark"]["color"] = "#"+str(self.config[entry][0]["dark"]["color"])

                # Set Background Colors for Labels
                if bgLabelLight in HTML_COLORS.keys(): # HTML HEXA COLOR
                    bgLabelLight = self.config[entry][0]["light"]["bglabel"]
                elif len(self.config[entry][0]["light"]["bglabel"]) in [3, 6, 8]:
                    bgLabelLight = self.config[entry][0]["light"]["bglabel"] = "#"+str(self.config[entry][0]["light"]["bglabel"])

                if bgLabelDark in HTML_COLORS.keys(): # HTML HEXA COLOR
                    bgLabelDark = self.config[entry][0]["dark"]["bglabel"]
                elif len(self.config[entry][0]["dark"]["bglabel"]) in [3, 6, 8]:
                    bgLabelDark = self.config[entry][0]["dark"]["bglabel"] = "#"+str(self.config[entry][0]["dark"]["bglabel"])

                # Set Background Colors for Forbidden Values (FV / Valeurs Interdites / VI)
                if bgVILight in HTML_COLORS.keys(): # HTML HEXA COLOR
                    bgVILight = self.config[entry][0]["light"]["bgvi"]
                elif len(self.config[entry][0]["light"]["bgvi"]) in [3, 6, 8]:
                    bgVILight = self.config[entry][0]["light"]["bgvi"] = "#"+str(self.config[entry][0]["light"]["bgvi"])

                if bgVIDark in HTML_COLORS.keys(): # HTML HEXA COLOR
                    bgVIDark = self.config[entry][0]["dark"]["bgvi"]
                elif len(self.config[entry][0]["dark"]["bgvi"]) in [3, 6, 8]:
                    bgVIDark = self.config[entry][0]["dark"]["bgvi"] = "#"+str(self.config[entry][0]["dark"]["bgvi"])

    def read_block(self, text:str)->(str, int) or (None, -1):
        """Returns a tuple:
        - the graphviz or dot block, if exists, and
        - a code integer to caracterize the command : 
            0 for a'grapvhiz' command, 
            1 if 'dot' command)
        or (None, None), if not a graphviz or dot command block"""
        blocks = [BLOCK_RE_TEX2SVG.search(text),
                  BLOCK_RE_VARTABLE.search(text)]
        for i in range(len(blocks)):
            if blocks[i] is not None:
                return blocks[i], i
        return None, -1

    def get_decalage(self, command:str, text:str)->int:
        """Renvoie le décalage (nombre d'espaces) où commencent les ``` dans la ligne ```command ...
        Cela suppose que le 'text' réellement la commande, ce qui est censé être le cas lros de l'utilisation de cette fonction
        """
        # command = 'tex2svg' or 'vartable' etc..
        i_command = text.find("```"+command)
        i_previous_linefeed = text[:i_command].rfind("\n")
        decalage = i_command - i_previous_linefeed-1
        return decalage

    def read_svg_from(self, filename:str):
        with open(filename, "r", encoding="utf-8") as f:
            svgList = f.readlines()
        return svg

    def to_file(self, content:str,filename:str):
        content = f"{content}"
        with open(filename, "w", encoding="utf-8") as f:
            f.write(content)

    def escape_chars(self, output):
        for c in ESC_CHAR:
            output = output.replace(c, ESC_CHAR[c])
        return output

    def get_preamble_postamble(self, inlineMode=True, packages=DEFAULT_PACKAGES)->tuple:
        if inlineMode:
            preamble = r"\documentclass{article}"+"\n"
            for package in packages:
                preamble += rf"\usepackage{{{package}}}"+"\n"
            preamble += r"""\begin{document}
\pagestyle{empty}
\begin{equation*}
"""
            postamble = r"""\end{equation*}
\end{document}
"""
        else: # block mode
            preamble = r"\documentclass{article}"+"\n"
            for package in packages:
                preamble += rf"\usepackage{{{package}}}"+"\n"
            preamble += r"""\begin{document}
\pagestyle{empty}
$$"""
            postamble = r"""$$
\end{document}
"""
        return (preamble, postamble)

    def formatSvg(self, svg, color, bgLabel, bgVI):
        svgList = svg.split("\n")
        if """<?xml version="1.0" encoding="UTF-8"?>""" == svgList[0]:
            # "<?xml ...>" element breaks Math Tables inside Admonitions, when at index 0 of list : let's remove it
            svgList = svgList[1:]
        svg = "\n".join(svgList)
        svg = svg.replace("fill=\"rgb(100%, 100%, 100%)\"", f"fill=\"{bgLabel}\"")
        svg = svg.replace("stroke=\"rgb(100%, 100%, 100%)\"", f"stroke=\"{bgVI}\"")
        svg = svg.replace("\"rgb(0%, 0%, 0%)\"", f"\"{color}\"")
        return svg

    def run(self, lines): # Preprocessors must extend markdown.Preprocessor
        """
        Each subclass of Preprocessor should override the `run` method, which
        takes the document as a list of strings split by newlines and returns
        the (possibly modified) list of lines.
        
        Match and generate dot code blocks.
        """
        dirName = "docs/tex2svg"
        dirName = dirName.lstrip("/")
        dirName = dirName.rstrip("/")
        # dirPath = f"./{dirName}"
        dirPath = f"{dirName}"
        os.makedirs(f"{dirPath}", exist_ok=True)
        text = "\n".join(lines)
        while 1:
            m, block_type = self.read_block(text)
            if not m:
                break
            else:
                salt = randint(1,100000)
                if block_type == TEX2SVG_COMMAND: # General Graphviz command
                    command = m.group('command')
                     # Whitelist command, prevent command injection.
                    if command not in SUPPORTED_COMMANDS:
                        raise Exception('Command not supported: %s' % command)
                    # text = self.escape_chars(text)
                    filename = m.group('filename')
                    decalage = self.get_decalage("graphviz "+command, text)
                    htmlHeader = " "*decalage+f"""<span class="tex2svg"></span>"""
                else: # VARTABLE command
                    # text = self.escape_chars(text)
                    filename = "noname.svg"
                    command = "vartable"
                    decalage = self.get_decalage(command, text)
                    htmlHeader = " "*decalage+f"""<span class="tex2svg vartable"></span>"""

                filetype = filename[filename.rfind('.')+1:]

                # RAW TEX2SVG BLOCK CONTENT
                content = m.group('content')
                tex = content

                try:
                    # bgcolor = self.config['bgcolor'][0]
                    bgcolor = "#"+DEFAULT_COLOR
                    preamble, postamble = self.get_preamble_postamble(True, DEFAULT_PACKAGES)
                    tex = preamble+tex+postamble

                    # Create Random Name
                    randomFilename = f"{dirPath}/tmp{salt}"
                    self.to_file(tex, f"{randomFilename}.tex")
                    print("randomFilename=", randomFilename)

                    # Export Tkz-tab to Svg
                    os.system(f"pdflatex -output-directory {dirPath} {randomFilename}.tex &> /dev/null")
                    os.system(f"pdfcrop {randomFilename}.pdf {randomFilename}_crop.pdf &> /dev/null")
                    os.system(f"pdf2svg {randomFilename}_crop.pdf {randomFilename}.svg &> /dev/null")

                    # Read Exported Svg File
                    encoding = "utf-8"
                    with open(f"{randomFilename}.svg", "rb") as f:
                        s = base64.b64encode(f.read())
                    svg = base64.b64decode(s).decode(f"{encoding}")

                    bgLight = self.config["vartable"][0]["light"]["color"]
                    bgLabelLight = self.config["vartable"][0]["light"]["bglabel"]
                    bgVILight = self.config["vartable"][0]["light"]["bgvi"]

                    bgDark = self.config["vartable"][0]["dark"]["color"]
                    bgLabelDark = self.config["vartable"][0]["dark"]["bglabel"]
                    bgVIDark = self.config["vartable"][0]["dark"]["bgvi"]

                    svgLight = self.formatSvg(svg, bgLight, bgLabelLight, bgVILight)
                    svgDark = self.formatSvg(svg, bgDark, bgLabelDark, bgVIDark)

                    # Create and Save three files : one for Light Mode, one for separation of the contiguous svgs, and one for Dark Mode
                    self.to_file(svgLight, f"{randomFilename}light.svg")
                    htmlSeparator = """<span id="separator"></span>\n"""
                    self.to_file(htmlSeparator, f"{dirPath}/separator.html")
                    self.to_file(svgDark, f"{randomFilename}dark.svg")

                    # Start of Popen
                    cmd_parts = [f"""cat {randomFilename}light.svg {dirPath}/separator.html {randomFilename}dark.svg"""]

                    i = 0
                    p = {}
                    for cmd_part in cmd_parts:
                        try:
                            cmd_part = cmd_part.strip()
                            if i == 0:
                                p[0]=subprocess.Popen(shlex.split(cmd_part),stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                            else:
                                p[i]=subprocess.Popen(shlex.split(cmd_part),stdin=p[i-1].stdout, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                            i += 1
                        except Exception as e:
                            err = str(e) + ' : ' + str(cmd_part)
                            return (
                                '<pre>Error : ' + err + '</pre>'
                                '<pre>' + content + '</pre>').split('\n')
                    (output, err) = p[i-1].communicate()
                    exit_code = p[0].wait()

                    svgGlobal = output.decode("utf-8")
                    svgList = svgGlobal.split(htmlSeparator)

                    svgLight = svgList[0]
                    svgDark = svgList[1]

                    # create IMG1 for Light Mode
                    base64Encoding = 'base64'
                    svgLight = svgLight.encode('utf-8')
                    svgLight = base64.b64encode(svgLight).decode('utf-8')
                    data_url_filetype = 'svg+xml'
                    data_path1 = "data:image/%s;%s,%s" % (
                        data_url_filetype,
                        base64Encoding,
                        svgLight)
                    img1 = " "*decalage+"![" + randomFilename + "Light" + "](" + data_path1 + "){.text2svg .light}"

                    # create IMG2 for Dark Mode
                    svgDark = svgDark.encode('utf-8')
                    svgDark = base64.b64encode(svgDark).decode('utf-8')
                    data_url_filetype = 'svg+xml'
                    data_path2 = "data:image/%s;%s,%s" % (
                        data_url_filetype,
                        base64Encoding,
                        svgDark)
                    img2 = " "*decalage+"![" + randomFilename + "Dark" + "](" + data_path2 + "){.tex2svg .dark}"

                    # Clean The Tmp Folder after
                    if not DEBUG:
                        os.remove(f"""{randomFilename}.tex""")
                        os.remove(f"""{randomFilename}.aux""")
                        os.remove(f"""{randomFilename}.log""")
                        os.remove(f"""{randomFilename}.pdf""")
                        os.remove(f"""{randomFilename}_crop.pdf""")
                        os.remove(f"""{randomFilename}.svg""")
                        if os.path.exists(f"""{randomFilename}dark.svg"""):
                            os.remove(f"""{randomFilename}dark.svg""")
                        if os.path.exists(f"""{randomFilename}light.svg"""):
                            os.remove(f"""{randomFilename}light.svg""")

                    # text = '%s\n%s\n%s\n%s' % (
                    #     text[:m.start()], img1, img2, text[m.end():])
                    text = f"""{text[:m.start()]!s}\n{htmlHeader!s}\n{img1!s}\n{img2!s}\n{text[m.end():]!s}"""
                except Exception as e:
                        return (
                            '<pre>Error : ' + str(e) + '</pre>'
                            '<pre>' + content + '</pre>').split('\n')
        return text.split("\n")

def makeExtension(*args, **kwargs):
    return MkdocsTex2SvgExtension(*args, **kwargs)



import os
import sys
from setuptools import setup

username = os.getenv('TWINE_USERNAME')
password = os.getenv('TWINE_PASSWORD')

VERSION = '0.0.58'
GIT_MESSAGE_FOR_THIS_VERSION ="""First Operational Version
"""

if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep build"):
        print("'build' is not installed.\nUse `pip install build`.\nExiting.")
        sys.exit()
    if os.system("pip freeze | grep twine"):
        print("'twine' is not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python -m build")
    if ((username is not None) and (password is not None)):
        os.system(f"python -m twine upload dist/* -u {username} -p {password}")
    else:
        os.system(f"python -m twine upload dist/*")
    print(f"You probably also want to git push project, create v{VERSION} tag, and push it too :\n")
    gitExport=input("Do you want to do it right now? [y(default) / n]")
    if gitExport=="y" or gitExport=="":
        os.system(f"git add . && git commit -m 'v{VERSION} : {GIT_MESSAGE_FOR_THIS_VERSION}' && git push")
        os.system(f"git tag -a {VERSION} -m 'v{VERSION}'")
        os.system(f"git push --tags")
    else:
        print("You may still consider adding the following new tag later on :")
        print(f"git tag -a {VERSION} -m 'v{VERSION}'")
        print(f"git push --tags")
    sys.exit()

setup(
    name="mkdocs-tex2svg",
    version=VERSION,
    packages=["mkdocs_tex2svg"],
    install_requires=['Markdown>=2.3.1'],
    author="Rodrigo Schwencke",
    author_email="rod2ik.dev@gmail.com",
    description="Converts (inline and/or block) LaTeX snippets in Mkdocs, as base64 SVGs, natively & dynamically compatible with Mkdocs Light & Dark Themes, directly from your Markdown",
    long_description_content_type="text/markdown",
    long_description="""Project Page : [rod2ik/mkdocs-tex2svg](https://gitlab.com/rod2ik/mkdocs-tex2svg)

This project is part of other mkdocs-related projects.  
If interested, please consider having a look at this page for a more complete list of configuration and examples, as well as all other mkdocs-related projects:

* https://eskool.gitlab.io/mkhack3rs/

This project is the work of :

* Rodrigo Schwencke at [rod2ik/mkdocs-tex2svg](https://gitlab.com/rod2ik/mkdocs-tex2svg)

In order to get it work with pip (python3) and mkdocs.

If you use python 2, please consider using the original extension instead.

Licence :

* Rodrigo Schwencke are [GPLv3+](https://opensource.org/licenses/GPL-3.0)""",
    keywords='mkdocs python markdown tex latex mathematics variation table svg',
    license="GPLv3+",
    url="https://gitlab.com/rod2ik/mkdocs-tex2svg.git",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Documentation',
        'Topic :: Text Processing',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows 10',
    ],
)



HTML_COLORS = {'aliceblue': '#f0f8ff',
 'antiquewhite': '#faebd7',
 'aqua': '#00ffff',
 'aquamarine': '#7fffd4',
 'azure': '#f0ffff',
 'beige': '#f5f5dc',
 'bisque': '#ffe4c4',
 'black': '#000000',
 'blanchedalmond': '#ffebcd',
 'blue': '#0000ff',
 'blueviolet': '#8a2be2',
 'brown': '#a52a2a',
 'burlywood': '#deb887',
 'cadetblue': '#5f9ea0',
 'chartreuse': '#7fff00',
 'chocolate': '#d2691e',
 'coral': '#ff7f50',
 'cornflowerblue': '#6495ed',
 'cornsilk': '#fff8dc',
 'crimson': '#dc143c',
 'cyan': '#00ffff',
 'darkblue': '#00008b',
 'darkcyan': '#008b8b',
 'darkgoldenrod': '#b8860b',
 'darkgray': '#a9a9a9',
 'darkgreen': '#006400',
 'darkgrey': '#a9a9a9',
 'darkkhaki': '#bdb76b',
 'darkmagenta': '#8b008b',
 'darkolivegreen': '#556b2f',
 'darkorange': '#ff8c00',
 'darkorchid': '#9932cc',
 'darkred': '#8b0000',
 'darksalmon': '#e9967a',
 'darkseagreen': '#8fbc8f',
 'darkslateblue': '#483d8b',
 'darkslategray': '#2f4f4f',
 'darkslategrey': '#2f4f4f',
 'darkturquoise': '#00ced1',
 'darkviolet': '#9400d3',
 'deeppink': '#ff1493',
 'deepskyblue': '#00bfff',
 'dimgray': '#696969',
 'dimgrey': '#696969',
 'dodgerblue': '#1e90ff',
 'firebrick': '#b22222',
 'floralwhite': '#fffaf0',
 'forestgreen': '#228b22',
 'fuchsia': '#ff00ff',
 'gainsboro': '#dcdcdc',
 'ghostwhite': '#f8f8ff',
 'gold': '#ffd700',
 'goldenrod': '#daa520',
 'gray': '#808080',
 'green': '#008000',
 'greenyellow': '#adff2f',
 'grey': '#808080',
 'honeydew': '#f0fff0',
 'hotpink': '#ff69b4',
 'indianred': '#cd5c5c',
 'indigo': '#4b0082',
 'ivory': '#fffff0',
 'khaki': '#f0e68c',
 'lavender': '#e6e6fa',
 'lavenderblush': '#fff0f5',
 'lawngreen': '#7cfc00',
 'lemonchiffon': '#fffacd',
 'lightblue': '#add8e6',
 'lightcoral': '#f08080',
 'lightcyan': '#e0ffff',
 'lightgoldenrodyellow': '#fafad2',
 'lightgray': '#d3d3d3',
 'lightgreen': '#90ee90',
 'lightgrey': '#d3d3d3',
 'lightpink': '#ffb6c1',
 'lightsalmon': '#ffa07a',
 'lightseagreen': '#20b2aa',
 'lightskyblue': '#87cefa',
 'lightslategray': '#778899',
 'lightslategrey': '#778899',
 'lightsteelblue': '#b0c4de',
 'lightyellow': '#ffffe0',
 'lime': '#00ff00',
 'limegreen': '#32cd32',
 'linen': '#faf0e6',
 'magenta': '#ff00ff',
 'maroon': '#800000',
 'mediumaquamarine': '#66cdaa',
 'mediumblue': '#0000cd',
 'mediumorchid': '#ba55d3',
 'mediumpurple': '#9370db',
 'mediumseagreen': '#3cb371',
 'mediumslateblue': '#7b68ee',
 'mediumspringgreen': '#00fa9a',
 'mediumturquoise': '#48d1cc',
 'mediumvioletred': '#c71585',
 'midnightblue': '#191970',
 'mintcream': '#f5fffa',
 'mistyrose': '#ffe4e1',
 'moccasin': '#ffe4b5',
 'navajowhite': '#ffdead',
 'navy': '#000080',
 'oldlace': '#fdf5e6',
 'olive': '#808000',
 'olivedrab': '#6b8e23',
 'orange': '#ffa500',
 'orangered': '#ff4500',
 'orchid': '#da70d6',
 'palegoldenrod': '#eee8aa',
 'palegreen': '#98fb98',
 'paleturquoise': '#afeeee',
 'palevioletred': '#db7093',
 'papayawhip': '#ffefd5',
 'peachpuff': '#ffdab9',
 'peru': '#cd853f',
 'pink': '#ffc0cb',
 'plum': '#dda0dd',
 'powderblue': '#b0e0e6',
 'purple': '#800080',
 'rebeccapurple': '#663399',
 'red': '#ff0000',
 'rosybrown': '#bc8f8f',
 'royalblue': '#4169e1',
 'saddlebrown': '#8b4513',
 'salmon': '#fa8072',
 'sandybrown': '#f4a460',
 'seagreen': '#2e8b57',
 'seashell': '#fff5ee',
 'sienna': '#a0522d',
 'silver': '#c0c0c0',
 'skyblue': '#87ceeb',
 'slateblue': '#6a5acd',
 'slategray': '#708090',
 'slategrey': '#708090',
 'snow': '#fffafa',
 'springgreen': '#00ff7f',
 'steelblue': '#4682b4',
 'tan': '#d2b48c',
 'teal': '#008080',
 'thistle': '#d8bfd8',
 'tomato': '#ff6347',
 'turquoise': '#40e0d0',
 'violet': '#ee82ee',
 'wheat': '#f5deb3',
 'white': '#ffffff',
 'whitesmoke': '#f5f5f5',
 'yellow': '#ffff00',
 'yellowgreen': '#9acd32'}